 function [w, testPrediction, trainPrediction] = testMetricLearning(param)
 
    
    addpath(genpath('/share/data/weak-label-seg/weakLabelMovies/src/util/dSP_5.1/'));
    addpath('/share/data/weak-label-seg/weakLabelMovies/src/util/superparsing-eccv10/Utilities');
    
    param.ucmPath =  '/share/data/weak-label-seg/data/LabelTransfer/datasets/datasetLMO/globalPb-mat';
    param.truthPath =  sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/SemanticLabels'); 
     
    param.numFeatureGroup = 20;
    param.numClass = 33;
     
    if isfield(param, 'threshold')    
        param.truthPath =  sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/SemanticLabels/SP_Desc_k%1.2f',  param.threshold);
        param.superpixelPath = sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/DataUCM/Descriptors/SP_Desc_k%1.2f',  param.threshold);
        param.potentialPath = sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/DataUCM/Base_%1.2f/SemanticLabels',  param.threshold);
        potentialLabelPath  =  sprintf('/share/data/weak-label-seg/weakLabelMovies/src/data/unary_t_%1.2f.mat', param.threshold);
         
    else
        param.truthPath =  sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/SemanticLabels/SP_Desc_k200');
        param.superpixelPath = sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/Data/Descriptors/SP_Desc_k200');
        param.potentialPath = sprintf('/share/data/weak-label-seg/data/SiftFlowDataset/Data/Base/SemanticLabels');
        potentialLabelPath  =  sprintf('/share/data/weak-label-seg/weakLabelMovies/src/data/unary_k_200.mat');
    end
    
    
    if strcmp(param.unaryNormType,  'metric_DSP_full')
        unaryDir =  dir( fullfile(param.potentialPath, 'probPer_Label_Jia*'   ) );
    else
        unaryDir =  dir( fullfile(param.potentialPath,   'probPerLabel*' ) );
    end
    param.unaryPotentialFolder = fullfile(param.potentialPath, unaryDir.name );
    
%% get train and test file lists

    display( 'Loading testFileList and trainFileList......' );
    HOME = '/share/data/weak-label-seg/data/SiftFlowDataset';
    HOMEIMAGES = fullfile(HOME,'Images');
    fileList = dir_recurse(fullfile(HOMEIMAGES,'*.*'),0);
    
    testSetFile = '/share/data/weak-label-seg/data/SiftFlowDataset/TestSet1.txt';
    testFiles = sort(importdata(testSetFile));
    testFiles = strrep(testFiles,'\',filesep);
    
    testMask = zeros(size(fileList))==1;
    for i = 1:length(testFiles)
        testMask(strcmp(testFiles{i},fileList))=1;
    end
    
    testFileList = fileList(testMask);    
    trainFileList = fileList(~testMask);
    
    
    if param.subsampleRatio<1;       
        
        trainFileList =  trainFileList(randperm(  length(trainFileList) , ceil(length(trainFileList) * param.subsampleRatio )));
    end
    
    
    trainingSize = length(trainFileList)
    
    display('Loading training samples......');
    trainingSamples =  loadSample(trainFileList,  param);  
    
    display('Loading testing samples......');
    testingSamples =  loadSample(testFileList, param);
    
    
    
        
     
    %% writing out puts for debugging
    
%     display('Writing features and observation......');
%     trainSamplePath = '/tmp/trainingSamples';
%     mkdir(trainSamplePath);    
%     WriteOutput_Jia(trainingSamples, trainSamplePath, 100);
%     
%     testSamplePath = '/tmp/testingSamples';
%     mkdir(testSamplePath);
%     WriteOutput_Jia(testingSamples, testSamplePath, 200);
     
%     return

    Params.ArmijoIterations = 50;
    Params.C = 1;
    Params.CRFEraseMessages = 0;
    Params.CRFIterations = 20;
    Params.CRFMPIterations = 1;
    Params.epsilon = 1.0;
    Params.p = 2;
    Params.ReuseMessagesForF = 1;


    %%b learning on training set

    task = 1;
    w_init = ones( param.numClass * param.numFeatureGroup, 1);

    display('Learning with latentStructuredPrediction......');
    tic
    % [w, Prediction] = structuredPrediction(Samples, Params, task, w_init);
    [wTrain, trainPrediction] = latentStructuredPrediction(trainingSamples, Params, task, w_init);
    toc

    %% inference on testing set
    task = 2;   

    display('Inference with latentStructuredPrediction......');

    tic
    [w, testPrediction] = latentStructuredPrediction(testingSamples, Params, task, wTrain);
    toc

    stop = 1;

     
%%
     
 
end

function  Samples = loadSample(fileList, param) 
       
        Samples = cell(length(fileList),1);
        
        numClass =  param.numClass;
        
        
        for i=1:  length(fileList)
            
            i
            
            [dirN objName] = fileparts(fileList{i});
            matName = [objName, '.mat'];
            
            truthFile = fullfile(param.truthPath, dirN, matName );
            truth = load(truthFile);
            
            unaryFile = fullfile(param.unaryPotentialFolder, dirN, matName );
            unary = load(unaryFile);
            
            numSp =  size(unary.probPerDescPerLabel, 1);
            
            Samples{i}.Regions = cell(numSp, 1);
             
            observation =  zeros(1, numSp );            
            observation(truth.index.sp) = truth.index.label;            
            Samples{i}.Observation = observation;
            
            for j= 1:numSp
        
                Samples{i}.Regions{j}.c_r = 1;
                Samples{i}.Regions{j}.VariableIndices = [j];
              %   Samples{i}.Regions{j}.VariableIndices = 1;
        
                Samples{i}.Regions{j}.Features = cell( param.numClass * param.numFeatureGroup, 1);    
             
                count = 0;
                for k=1:numClass      
                    for l=1:param.numFeatureGroup
                        count = count+1;
                        Samples{i}.Regions{j}.Features{count}.r = count; 
                        pot = sparse(numClass, 1 );   
                        pot(k) = unary.probPerDescPerLabel(j,k,l);            
                        Samples{i}.Regions{j}.Features{count}.pot = pot;
                    end
                end                

                loss = ones(1, numClass );
                if observation(j)>0
                    loss( observation(j) )  =  0;
                end
                                
                Samples{i}.Regions{j}.Loss = loss;   
                Samples{i}.Regions{j}.Parents = [];
            end
            
            Samples{i}.VariableCardinalities = ones(1, numSp)*numClass;

            Samples{i}.MachineIDs = ones(1, numSp)*2;  %ignored when using matlab!!!

        end
    
        
 end
    